@@ -671,9 +671,11 @@
 
 
 
-  if(is.na(spseqs) && !is.na(dualseqs )){
+  if(any(is.na(spseqs)) && !is.na(dualseqs )){
     return(c(paste0(dualseqs[1], dualseqs[2]),paste0(dualseqs[1], dualseqs[3])) )
-  }else if(!any(is.na(spseqs))) return(spseqs) else stop(paste("Unknown sample index name:", spid))
+  }else{
+    if(!any(is.na(spseqs))) return(spseqs) else return(NA)
+  }
 }
 
 .rstest <- function(r, coef) r * (1 + 1/r)^(1 + coef)
@@ -695,7 +697,7 @@
 
   nobs <- length(obs)
   if(nobs < .min_mySGT_input_size){
-    cat(sprintf("Warning: not enough element in the observation array : %d < %d.\nThe `Rescued' element in the returned object will be set to NA.\n",  nobs, .min_mySGT_input_size))
+    cat("Note: not enough cells in the data for performing cell rescuing.\n")
     return(NA)
   }else{
       if(nobs != length(times)) stop("The oservation array doesn't match the frequency array.")
@@ -844,10 +846,21 @@
     stop(sprintf("ERROR: no known cell barcode set was found for the data set. The highest percentage of cell-barcode matched reads is %.1f%%\n", max.cell.good*100.))
 }
 
-library(Matrix)
+.read.sparse.mat.by.genes <- function (fn, genes){
+  mtxrows <- read.delim(paste0(fn,".spmtx"), skip=2, header=F, sep=' ')
+  coln <- read.delim(paste0(fn, ".BCtab"), stringsAsFactors=F, header=F)$V1
+  rown <- read.delim(paste0(fn, ".GENEtab"), stringsAsFactors=F, header=F)$V1
+  out.gene.idx <- match(rown ,genes )
+  gene.ranks <- out.gene.idx[as.numeric(mtxrows[,1])]
+  cell.ranks <- as.numeric(mtxrows[,2])
+  ret <- Matrix::sparseMatrix(i=gene.ranks, j=cell.ranks , x=as.numeric(mtxrows[,3]))
+  rownames(ret) <- genes 
+  colnames(ret) <- coln 
+  ret
+}
 .read.sparse.mat <- function (fn){
   #cat("Loading matrix from",fn,"\n")
-  mtx <- readMM(paste0(fn, ".spmtx"))
+  mtx <- Matrix::readMM(paste0(fn, ".spmtx"))
   if(file.size(paste0(fn, ".BCtab"))>0){
     coln <- read.delim(paste0(fn, ".BCtab"), stringsAsFactors=F, header=F)$V1
     colnames(mtx) <- coln
@@ -921,7 +934,6 @@ library(Matrix)
     }
     Old_bs_one <- bcsize_one
   }
-  #if(5000 <= all.steps.len)print("WARNING: SMALL STEPS >= 5000!")
 
   Old_bs_one <- NA
   M50.in.loop.K <- 0
@@ -1080,9 +1092,12 @@ library(Matrix)
 .load.one.scSample <- function( BAM.name, FC.gene.ids, sample.no, use.meta.features, annot.tab, umi.cutoff){
   set.seed(0)
   fname <- sprintf("%s.scRNA.%03d", BAM.name, sample.no)
-  #cat("Loading high-conf matrix from '",fname,"'\n")
-  cat("Process the UMI table for sample",sample.no,"...\n")
+  cat("Perform cell rescuing for sample",sample.no,"...\n")
   highconf <- as.matrix(.read.sparse.mat(paste0(fname,".HighConf")))
+  raw.fname <- paste0(fname,".RawOut.spmtx")
+  if(file.exists(raw.fname)){
+    rawout <- .read.sparse.mat.by.genes(paste0(fname,".RawOut"), FC.gene.ids)
+  }else rawout <- NULL
   rescued <- NA
   if(is.null(umi.cutoff)) rescued <- .cellCounts.rescue(BAM.name, FC.gene.ids, sample.no)
 
@@ -1100,12 +1115,15 @@ library(Matrix)
     ret[rownames(highconf), colnames(highconf) ] <- highconf
     if(!any(is.na(rescued)))ret[rownames(rescued), colnames(rescued) ] <- rescued
 
-    list(Counts=ret, HighConfidneceCell=colnames(ret) %in% colnames(highconf))
+    retc<- list(Counts=ret, HighConfidneceCell=colnames(ret) %in% colnames(highconf))
+    if(!is.null(rawout)) retc[["ExcludedCells"]] <- rawout[,!( colnames(rawout) %in% colnames(ret) )]
   }else{
     fcmat <- .match.exons(annot.tab,highconf)
     rownames(fcmat)<-NULL
-    list(Counts=fcmat, HighConfidneceCell=rep(T, ncol(highconf))) 
+    retc<- list(Counts=fcmat, HighConfidneceCell=rep(T, ncol(highconf)))
+    if(!is.null(rawout))retc[["ExcludedCells"]] <- rawout[,!( colnames(rawout) %in% colnames(fcmat) )]
   }
+  return(retc)
 }
 
 .load.all.scSamples <- function( BAM.name, FC.gene.ids, use.meta.features, annot.tab, umi.cutoff){
@@ -1198,7 +1216,7 @@ library(Matrix)
 .SCRNA_FASTA_SPLIT2 <- "|Rsd:cCounts:1mFQ|"
 
 .validate.sample.sheet <- function(sheet, input.mode){
-  if(is.na(sheet) || is.null(sheet)) stop("A sample sheet must be provided.")
+  if(any(is.na(sheet)) || is.null(sheet)) stop("A sample sheet must be provided.")
   if(input.mode == "BCL"){
     if(nrow(sheet)<1) stop("The sample sheet cannot be empty.")
     if(!("Lane" %in% colnames(sheet) && "IndexSetName" %in% colnames(sheet) && "SampleName" %in% colnames(sheet) && "InputDirectory" %in% colnames(sheet)))
@@ -1217,7 +1235,7 @@ library(Matrix)
   return(sheet)
 }
 
-cellCounts <- function( index, sample, input.mode = "BCL", cell.barcode = NULL, nsubreads = 15, minVotes = 1, maxMismatches = 10, minMappedLength = 1, annot.inbuilt = "mm39", annot.ext = NULL, isGTFAnnotationFile = FALSE, GTF.featureType = "exon", GTF.attrType = "gene_id", useMetaFeatures = TRUE, umi.cutoff = NULL, nthreads = 10, nBestLocations = 1, uniqueMapping = FALSE){
+cellCounts <- function( index, sample, input.mode = "BCL", cell.barcode = NULL, nsubreads = 15, minVotes = 1, maxMismatches = 10, minMappedLength = 1, annot.inbuilt = "mm39", annot.ext = NULL, isGTFAnnotationFile = FALSE, GTF.featureType = "exon", GTF.attrType = "gene_id", useMetaFeatures = TRUE, umi.cutoff = NULL, nthreads = 10, nBestLocations = 1, uniqueMapping = FALSE, reportExcludedBarcodes = FALSE){
   maxDiffToTopVotes=2
   onlyDetectBarcode=FALSE
   has.error <- FALSE
@@ -1225,9 +1243,10 @@ cellCounts <- function( index, sample, input.mode = "BCL", cell.barcode = NULL,
   minVotesPerRead <- minVotes
   subreadsPerRead <- nsubreads
   unique.mapping <- uniqueMapping
-  .remove.temp.files <- T
 
   index <- .check_and_NormPath(index, mustWork=F, opt="index name")
+  index.file.1 <- paste0(index, ".00.b.array")
+  if(!file.exists(index.file.1))stop(sprintf("Error: index '%s' is not found.", index))
   if(!is.null(umi.cutoff)){
     umi.cutoff <- as.numeric(umi.cutoff)
     if(umi.cutoff < 0.0) stop("UMI cutoff must be a positive number.")
@@ -1250,7 +1269,6 @@ cellCounts <- function( index, sample, input.mode = "BCL", cell.barcode = NULL,
     if(input.mode == "FASTQ-dir") sample.info.idx <- .scan.fastq.dir(sample)
     #print(sample.info.idx)
     if(!("BarcodeUMIFile" %in%  colnames(sample.info.idx) && "ReadFile" %in%  colnames(sample.info.idx) )) stop("You need to provide BC+UMI and Genomic sequence files")
- 
     combined.fastq.names <- ""
     for(rowi in 1:nrow(sample.info.idx)){
       df1 <- as.character(sample.info.idx[rowi,"BarcodeUMIFile"])
@@ -1271,7 +1289,7 @@ cellCounts <- function( index, sample, input.mode = "BCL", cell.barcode = NULL,
     }
 
      .index.names.to.sheet.FASTQ.mode(sample.info.idx, cc.sample.sheet.path)
-    opt <- c("--inputMode","FASTQ","--cellBarcodeFile", cell.barcode,"--dataset", combined.fastq.names, "--sampleSheetFile", cc.sample.sheet.path, "--index", index, "--annotation", ann, "--geneIdColumn", GTF.attrType, "--annotationType", GTF.featureType, "--threads", nthreads, "--output", temp.file.prefix, "--maxMismatch", maxMismatchBases, "--minVotesPerRead", minVotesPerRead, "--subreadsPerRead", subreadsPerRead, "--reportedAlignmentsPerRead", nBestLocations, "--maxDiffToTopVotes", maxDiffToTopVotes, "--minMappedLength", minMappedLength, "--umiCutoff",  ifelse(is.null(umi.cutoff), -999, umi.cutoff))
+    opt <- c("--inputMode","FASTQ","--cellBarcodeFile", cell.barcode,"--reportExcludedBarcodes",as.numeric(reportExcludedBarcodes),"--dataset", combined.fastq.names, "--sampleSheetFile", cc.sample.sheet.path, "--index", index, "--annotation", ann, "--geneIdColumn", GTF.attrType, "--annotationType", GTF.featureType, "--threads", nthreads, "--output", temp.file.prefix, "--maxMismatch", maxMismatchBases, "--minVotesPerRead", minVotesPerRead, "--subreadsPerRead", subreadsPerRead, "--reportedAlignmentsPerRead", nBestLocations, "--maxDiffToTopVotes", maxDiffToTopVotes, "--minMappedLength", minMappedLength, "--umiCutoff",  ifelse(is.null(umi.cutoff), -999, umi.cutoff))
     if(isGTFAnnotationFile)opt <- c(opt, "--isGTFannotation")
     if(!unique.mapping)opt <- c(opt, "--reportMultiMappingReads")
 
@@ -1285,9 +1303,12 @@ cellCounts <- function( index, sample, input.mode = "BCL", cell.barcode = NULL,
       raw.fc.annot<-read.delim(annot.file, header=T, stringsAsFactors=F)
       some.results <- .load.all.scSamples(temp.file.prefix, as.character(raw.fc.annot$GeneID), useMetaFeatures, raw.fc.annot, umi.cutoff)
 
+      fc[["counts"]] <- list()
+      if(reportExcludedBarcodes)fc[["counts.excluded.barcodes"]] <- list()
       for(spi in 1:nrow(some.results[["Sample.Table"]])){
         samplename <- as.character(some.results[["Sample.Table"]][["SampleName"]][spi])
         fc[["counts"]][[samplename]] <- some.results[[sprintf("Sample.%d", spi)]][["Counts"]] # only one sample.
+        if(reportExcludedBarcodes)fc[["counts.excluded.barcodes"]][[samplename]] <- some.results[[sprintf("Sample.%d", spi)]][["ExcludedCells"]] # only one sample.
         if(is.null(umi.cutoff))fc[["cell.confidence"]][[samplename]] <- some.results[[sprintf("Sample.%d", spi)]][["HighConfidneceCell"]]
       }
       df.sample.info <- .extract.sample.table.cols(NA,some.results,input.mode="fastq", umi.cutoff=umi.cutoff)
@@ -1298,6 +1319,7 @@ cellCounts <- function( index, sample, input.mode = "BCL", cell.barcode = NULL,
 
     dirs <- unique( sample.info.idx$InputDirectory )
     fc[["counts"]] <- list()
+    if(reportExcludedBarcodes)fc[["counts.excluded.barcodes"]] <- list()
   
     for(dirname in dirs) .check_and_NormPath(dirname, mustWork=TRUE, "InputDirectory in sample.info.idx") # check files before the slow mapping/counting step.
 
@@ -1322,7 +1344,7 @@ cellCounts <- function( index, sample, input.mode = "BCL", cell.barcode = NULL,
       is_dual_index <- .index.names.to.sheet.raw.dir.mode(dirname, sample.info.idx, cc.sample.sheet.path)
       generate.scRNA.BAM <- TRUE
 
-      opt <- c("--cellBarcodeFile", cell.barcode,"--dataset", dirname, "--sampleSheetFile", cc.sample.sheet.path, "--index", index, "--annotation", ann, "--geneIdColumn", GTF.attrType, "--annotationType", GTF.featureType, "--threads", nthreads, "--output", temp.file.prefix, "--maxMismatch", maxMismatchBases, "--minVotesPerRead", minVotesPerRead, "--subreadsPerRead", subreadsPerRead, "--maxDiffToTopVotes",maxDiffToTopVotes, "--minMappedLength", minMappedLength, "--umiCutoff", ifelse(is.null(umi.cutoff), -999, umi.cutoff))
+      opt <- c("--cellBarcodeFile", cell.barcode,"--reportExcludedBarcodes",as.numeric(reportExcludedBarcodes),"--dataset", dirname, "--sampleSheetFile", cc.sample.sheet.path, "--index", index, "--annotation", ann, "--geneIdColumn", GTF.attrType, "--annotationType", GTF.featureType, "--threads", nthreads, "--output", temp.file.prefix, "--maxMismatch", maxMismatchBases, "--minVotesPerRead", minVotesPerRead, "--subreadsPerRead", subreadsPerRead, "--maxDiffToTopVotes",maxDiffToTopVotes, "--minMappedLength", minMappedLength, "--umiCutoff", ifelse(is.null(umi.cutoff), -999, umi.cutoff))
       if(isGTFAnnotationFile)opt <- c(opt, "--isGTFannotation")
       if(!unique.mapping)opt <- c(opt, "--reportMultiMappingReads")
 
@@ -1339,6 +1361,7 @@ cellCounts <- function( index, sample, input.mode = "BCL", cell.barcode = NULL,
         for(spi in 1:nrow(some.results[["Sample.Table"]])){
           samplename <- as.character(some.results[["Sample.Table"]][["SampleName"]][spi])
           fc[["counts"]][[samplename]] <- some.results[[sprintf("Sample.%d", spi)]][["Counts"]] # only one sample.
+          if(reportExcludedBarcodes)fc[["counts.excluded.barcodes"]][[samplename]] <- some.results[[sprintf("Sample.%d", spi)]][["ExcludedCells"]] # only one sample.
           if(is.null(umi.cutoff))fc[["cell.confidence"]][[samplename]] <- some.results[[sprintf("Sample.%d", spi)]][["HighConfidneceCell"]]
         }
         stt <- .extract.sample.table.cols(full_dirname,some.results, umi.cutoff=umi.cutoff)
@@ -1360,7 +1383,7 @@ cellCounts <- function( index, sample, input.mode = "BCL", cell.barcode = NULL,
       BAM.names <- paste(sample.info.idx$BAMFile[ as.character(sample.info.idx$SampleName) == this.sample ], collapse=.SCRNA_FASTA_SPLIT1)
       generate.scRNA.BAM <- TRUE
 
-      opt <- c("--inputMode","BAM","--cellBarcodeFile", cell.barcode,"--dataset", BAM.names, "--sampleSheetFile", cc.sample.sheet.path, "--index", index, "--annotation", ann, "--geneIdColumn", GTF.attrType, "--annotationType", GTF.featureType, "--threads", nthreads, "--output", temp.file.prefix, "--maxMismatch", maxMismatchBases, "--minVotesPerRead", minVotesPerRead, "--subreadsPerRead", subreadsPerRead, "--maxDiffToTopVotes",maxDiffToTopVotes, "--minMappedLength", minMappedLength, "--umiCutoff", ifelse(is.null(umi.cutoff), -999, umi.cutoff))
+      opt <- c("--inputMode","BAM","--cellBarcodeFile", cell.barcode, "--reportExcludedBarcodes",as.numeric(reportExcludedBarcodes),"--dataset", BAM.names, "--sampleSheetFile", cc.sample.sheet.path, "--index", index, "--annotation", ann, "--geneIdColumn", GTF.attrType, "--annotationType", GTF.featureType, "--threads", nthreads, "--output", temp.file.prefix, "--maxMismatch", maxMismatchBases, "--minVotesPerRead", minVotesPerRead, "--subreadsPerRead", subreadsPerRead, "--maxDiffToTopVotes",maxDiffToTopVotes, "--minMappedLength", minMappedLength, "--umiCutoff", ifelse(is.null(umi.cutoff), -999, umi.cutoff))
       if(isGTFAnnotationFile)opt <- c(opt, "--isGTFannotation")
       if(!unique.mapping)opt <- c(opt, "--reportMultiMappingReads")
 
@@ -1371,18 +1394,23 @@ cellCounts <- function( index, sample, input.mode = "BCL", cell.barcode = NULL,
 
       annot.file <- paste0(temp.file.prefix,".Annot")
       if(file.exists(annot.file)){
+        fc[["counts"]] <- list()
+        if(reportExcludedBarcodes)fc[["counts.excluded.barcodes"]] <- list()
+
         raw.fc.annot<-read.delim(annot.file, header=T, stringsAsFactors=F)
         some.results <- .load.all.scSamples(temp.file.prefix, as.character(raw.fc.annot$GeneID), useMetaFeatures, raw.fc.annot, umi.cutoff)
         for(spi in 1:nrow(some.results[["Sample.Table"]])){
           samplename <- as.character(some.results[["Sample.Table"]][["SampleName"]][spi])
           fc[["counts"]][[samplename]] <- some.results[[sprintf("Sample.%d", spi)]][["Counts"]] # only one sample.
+          if(reportExcludedBarcodes)fc[["counts.excluded.barcodes"]][[samplename]] <- some.results[[sprintf("Sample.%d", spi)]][["ExcludedCells"]] # only one sample.
           if(is.null(umi.cutoff))fc[["cell.confidence"]][[samplename]] <- some.results[[sprintf("Sample.%d", spi)]][["HighConfidneceCell"]]
         }
         df.sample.info <- rbind(df.sample.info,.extract.sample.table.cols(NA,some.results, input.mode="bam", umi.cutoff=umi.cutoff))
       }else has.error<-T
     }
   }
-  .del.temp.files(substr(temp.file.prefix,4,99))
+  if(T).del.temp.files(substr(temp.file.prefix,4,99)) else warning("NOT DELETING TEMP FILES !!!!")
+
   fc[["annotation"]] <- raw.fc.annot
   fc[["sample.info"]] <- df.sample.info
   if(has.error) { 
