@@ -734,7 +734,7 @@ int convert_BAM_binary_to_SAM( SamBam_Reference_Info * chro_table, char * bam_bi
 					memcpy(&tagval,  bam_bin + flex_ptr, type_bytes);
 					long long printv = is_signed?tagval:( (unsigned int) tagval );
 					#ifdef __MINGW32__
-					sam_ptr += sprintf(sam_txt + sam_ptr, "%I64d\t", printv);
+					sam_ptr += sprintf(sam_txt + sam_ptr, "%" PRId64 "\t", printv);
 					#else
 					sam_ptr += sprintf(sam_txt + sam_ptr, "%lld\t", printv);
 					#endif
@@ -988,7 +988,7 @@ int PBam_chunk_gets(char * chunk, int *chunk_ptr, int chunk_limit, SamBam_Refere
 	//fprintf(stderr, "HN_TAG=%d\n", nh_val	);
 
 	#ifdef __MINGW32__
-	int plen = snprintf(buff, buff_len-1, "%s\t%u\t%s\t%u\t%d\t%s\t%s\t%u\t%I64d\t%s\t%s%s\n%c", aln -> read_name, aln -> flags , chro_name, chro_offset, aln -> mapping_quality, cigar, mate_chro_name, mate_chro_offset, templete_length, aln -> sequence , aln -> seq_quality, extra_tags, 0);
+	int plen = snprintf(buff, buff_len-1, "%s\t%u\t%s\t%u\t%d\t%s\t%s\t%u\t%" PRId64 "\t%s\t%s%s\n%c", aln -> read_name, aln -> flags , chro_name, chro_offset, aln -> mapping_quality, cigar, mate_chro_name, mate_chro_offset, templete_length, aln -> sequence , aln -> seq_quality, extra_tags, 0);
 	#else
 	int plen = snprintf(buff, buff_len-1, "%s\t%u\t%s\t%u\t%d\t%s\t%s\t%u\t%lld\t%s\t%s%s\n%c", aln -> read_name, aln -> flags , chro_name, chro_offset, aln -> mapping_quality, cigar, mate_chro_name, mate_chro_offset, templete_length, aln -> sequence , aln -> seq_quality, extra_tags, 0);
 	#endif
@@ -2376,7 +2376,7 @@ void SamBam_writer_sort_bins_to_BAM(SamBam_Writer * writer){
 	SAMBAM_renew_BAItabs;
 
 	int chro_no = (int)(current_min >> 32);
-	int old_chro_no = -1, last_written_BAI_chro = -1;
+	int old_chro_no = -1;
 
 	int wlen = fwrite("BAI\1", 4, 1, writer -> BAI_fp);
 	if(wlen < 1) assert(wlen >0);
@@ -2411,7 +2411,6 @@ void SamBam_writer_sort_bins_to_BAM(SamBam_Writer * writer){
 		if(old_chro_no >=0 && chro_no != old_chro_no){ // if there is old_chro && if has to write. Note: old_chro_no<0 means we reached the "unmapped" part of the sorted temp files
 			//SUBREADprintf("\n====== W1CHR %d =====\n", old_chro_no);
 			SamBam_write_BAI_for_1chr(writer, &bin_chunks_table, &bins_list, &win16k_list);
-			last_written_BAI_chro = old_chro_no;
 			SAMBAM_write_empty_chros(old_chro_no +1, chro_no < 0?n_ref:chro_no);
 			SAMBAM_renew_BAItabs;
 			SAMBAM_reset_sorting_writer;
